# coding: utf-8

"""
    Fiddle Options Platform

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import
from datetime import datetime, date
import copy
import unittest

from mercury.client.model import Position

from mercury import FiddleApi
from mercury import Fiddle
from tests.utils import TestUtils, create_api_client


class TestFiddleApi(unittest.TestCase):
    """FiddleApi unit test stubs"""

    def setUp(self):
        self.api = FiddleApi(api_client=create_api_client())
        self.test_utils = TestUtils()
        self._fiddle = None
        # test user
        self.user_id = 'unit'

    def tearDown(self):
        if self._fiddle:
            self._delete_test_fiddle()

    def _fetch_fiddle(self, iid):
        fid = self.api.get_fiddle(iid)
        self.assertIsNotNone(fid)
        return fid

    def _create_test_fiddle(self):
        # refactor: not ideal
        opening_date = date(2018, 4, 9)
        long_put_butterfly = self.test_utils.\
            create_long_put_butterfly(opening_date)

        fiddle = Fiddle(user_id=self.user_id,
                        title='Long Butterfly April 9th test Python',
                        position=Position(trades=[long_put_butterfly]))
        data = {'fiddle': fiddle}
        self._fiddle = self.api.save_fiddle(self.user_id, **data)

        # test if the fiddle is the same as sent, but first set the id
        fiddle.id = self._fiddle.id

        self.assertEqual(self._fiddle, fiddle)
        return self._fiddle

    def _create_complex_fiddle(self):
        # refactor: not ideal
        opening_date = date(2018, 4, 9)

        # create trades
        butterfly = self.test_utils.\
            create_long_put_butterfly(opening_date)

        single_put = self.test_utils.create_single_option(opening_date)

        # initialize Position with butterfly
        position = Position(trades=[butterfly])

        # add single_put
        position.add_trade(single_put)

        fiddle = Fiddle(user_id=self.user_id,
                        title='Long Butterfly + Single April 9th test Python',
                        position=position)
        return fiddle

    def _create_really_complex_fiddle(self):
        # refactor: not ideal
        opening_date = date(2018, 4, 9)

        # create trades
        butterfly = self.test_utils.\
            create_long_put_butterfly(opening_date)

        single_put = self.test_utils.create_single_option(opening_date)

        vertical_opening_date = date(2018, 4, 25)
        vertical_spread = self.test_utils.create_vertical_spread(vertical_opening_date)

        # initialize Position with butterfly
        position = Position(trades=[butterfly])

        # add single_put
        position.add_trade(single_put)

        position.add_trade(vertical_spread)

        assert len(position.trades) == 3

        fiddle = Fiddle(user_id=self.user_id,
                        title='Long Butterfly + Single + Vertical April 9th test Python',
                        position=position)
        return fiddle

    def _delete_test_fiddle(self):
        # refactor: not ideal
        if self._fiddle:
            ret = self.api.delete_fiddle(self._fiddle.id)
            # todo: error 500
            self._fiddle = None

    # @unittest.expectedFailure
    def test_delete_fiddle(self):
        """Test case for delete_fiddle

        Deletes Fiddle with specified id  # noqa: E501
        """
        self._create_test_fiddle()
        self.assertIsNotNone(self._fiddle)
        created_id = self._fiddle.id
        # self._delete_test_fiddle()
        # todo: get_fiddle(created_id) check response

    def test_get_fiddle(self):
        """Test case for get_fiddle

        Returns the Fiddle with specified id  # noqa: E501
        """
        fid = self._create_test_fiddle()
        time_start = datetime.now()
        print('Loading Fiddle...')
        fiddle = self._fetch_fiddle(fid.id)
        load_time = (datetime.now() - time_start)
        print('Loaded fiddle %s' % fiddle)
        print('Loaded in %s' % load_time)

        self.assertIsNotNone(fiddle, "No fiddle returned")
        self.assertEqual(self._fiddle, fiddle, "Created fiddle not equal to fetched one")
        self.assertLess(load_time.seconds, 5, "Fiddle loaded in more than 5 seconds.")

    @unittest.expectedFailure
    def test_get_unknown_fiddle(self):
        self._fetch_fiddle('peroorep')

    def test_get_fiddle_list(self):
        """Test case for get_fiddle_list

        Returns the list of Fiddles for the specified user  # noqa: E501
        """
        pass

    def test_get_position_legs(self):
        """Test case for get_position_legs

        Returns all options contracts that the specified options position is consisted of  # noqa: E501
        """
        pass

    def test_get_trade_from_tos_string(self):
        """Test case for get_trade_from_tos_string

        Creates a list of positions from a list of trades specified in ThinkOrSwim format  # noqa: E501
        """
        params = {'trades': [
                             'BUY +7 VERTICAL SPX 100 15 JUN 18 2730/2725 PUT @1.75 LMT',
                             'SELL -8 VERTICAL SPX 100 15 JUN 18 2735/2725 PUT @3.60 LMT',
                             'BUY +10 STRADDLE SPX 100 15 JUN 18 2735 CALL/PUT @56.80 LMT',
                             'SELL -10 STRADDLE SPX 100 15 JUN 18 2735 CALL/PUT @56.70 LMT',
                             'BUY +10 STRANGLE SPX 100 15 JUN 18 2740/2735 CALL/PUT @53.10 LMT',
                             'SELL -1 STRANGLE SPX 100 15 JUN 18 2730/2735 CALL/PUT @59.30 LMT',
                             'BUY +10 CONDOR SPX 100 15 JUN 18 2740/2735/2730/2725 PUT @.40 LMT',
                             'SELL -10 CONDOR SPX 100 15 JUN 18 2740/2735/2730/2725 PUT @.35 LMT',
                             'BUY +10 IRON CONDOR SPX 100 15 JUN 18 2745/2750/2740/2735 CALL/PUT @4.65 LMT',
                             'SELL -10 IRON CONDOR SPX 100 15 JUN 18 2740/2745/2735/2730 CALL/PUT @4.65 LMT',
                             'BUY +16 BUTTERFLY SPX 100 15 JUN 18 2735/2730/2725 PUT @.20 LMT',
                             'SELL -11 BUTTERFLY SPX 100 15 JUN 18 2730/2735/2740 CALL @.10 LMT'
                             ],
                  "opening_date": date(2018, 4, 9)}
        trades = self.api.get_trade_from_tos_string(**params)

    def test_create_fiddle(self):
        """Test case for save_fiddle

        Saves a new Fiddle or updates an existing one  # noqa: E501
        """
        fiddle = self._create_test_fiddle()
        self.assertIsNotNone(fiddle)
        print('Saved fiddle %s' % self._fiddle)

    def test_create_save_complex_fiddle(self):
        fiddle = self._create_complex_fiddle()
        data = {'fiddle': fiddle}
        self._fiddle = self.api.save_fiddle(self.user_id, **data)

        # test if the fiddle is the same as sent, but first set the id
        fiddle.id = self._fiddle.id
        self.maxDiff = 99999
        self.assertDictEqual(self._fiddle.to_dict(), fiddle.to_dict())

    def test_create_save_really_complex_fiddle(self):
        fiddle = self._create_really_complex_fiddle()
        data = {'fiddle': fiddle}
        self._fiddle = self.api.save_fiddle(self.user_id, **data)

    def test_update_fiddle(self):
        original_fiddle = self._create_test_fiddle()
        # Make a copy to modify
        updated_fiddle = copy.copy(original_fiddle)

        self.assertEqual(original_fiddle, updated_fiddle)

        # Make changes
        updated_fiddle.title = "Updated Title !"

        # response_fiddle = self.api.save_fiddle_object(self.user_id, updated_fiddle)
        response_fiddle = self.api.save_fiddle(self.user_id, **{'fiddle': updated_fiddle})
        fetched_fid = self._fetch_fiddle(original_fiddle.id)

        self.assertNotEqual(original_fiddle, updated_fiddle)
        self.assertEqual(updated_fiddle, fetched_fid)
        self.assertEqual(response_fiddle, fetched_fid)
        self.assertNotEqual(original_fiddle, fetched_fid)


if __name__ == '__main__':
    unittest.main()
